@@ -22,9 +22,9 @@ $compiled = {}
 $:.replace([Dir.pwd])
 require 'rbconfig'
 
-srcdir = File.expand_path("../..", __FILE__)
+srcdir = File.dirname(File.dirname(__FILE__))
 
-$:.unshift(srcdir, srcdir+"/lib")
+$:.unshift(srcdir, File.expand_path("lib", srcdir))
 
 $topdir = "."
 $top_srcdir = srcdir
@@ -129,6 +129,7 @@ def extmake(target)
         then
 	  ok = false
           init_mkmf
+          $defs << "-DRUBY_EXPORT" if $static
 	  Logging::logfile 'mkmf.log'
 	  rm_f makefile
 	  if File.exist?($0 = "#{$srcdir}/makefile.rb")
@@ -158,7 +159,7 @@ def extmake(target)
     end
     args = sysquote($mflags)
     unless $destdir.to_s.empty? or $mflags.include?("DESTDIR")
-      args << sysquote("DESTDIR=" + relative_from($destdir, "../"+prefix))
+      args += [sysquote("DESTDIR=" + relative_from($destdir, "../"+prefix))]
     end
     if $static
       args += ["static"] unless $clean
@@ -350,12 +351,30 @@ end unless $extstatic
 
 ext_prefix = "#{$top_srcdir}/ext"
 exts = $static_ext.sort_by {|t, i| i}.collect {|t, i| t}
-exts |= $extension if $extension
-exts |= Dir.glob("#{ext_prefix}/*/**/extconf.rb").collect {|d|
-  d = File.dirname(d)
-  d.slice!(0, ext_prefix.length + 1)
-  d
-}.sort unless $extension
+if $extension
+  exts |= $extension.select {|d| File.directory?("#{ext_prefix}/#{d}")}
+else
+  withes, withouts = %w[--with --without].collect {|w|
+    if not (w = %w[-extensions -ext].collect {|opt|arg_config(w+opt)}).any?
+      proc {false}
+    elsif (w = w.grep(String)).empty?
+      proc {true}
+    else
+      w.collect {|opt| opt.split(/,/)}.flatten.method(:any?)
+    end
+  }
+  cond = proc {|ext|
+    cond1 = proc {|n| File.fnmatch(n, ext, File::FNM_PATHNAME)}
+    withes.call(&cond1) or !withouts.call(&cond1)
+  }
+  exts |= Dir.glob("#{ext_prefix}/*/**/extconf.rb").collect {|d|
+    d = File.dirname(d)
+    d.slice!(0, ext_prefix.length + 1)
+    d
+  }.find_all {|ext|
+    with_config(ext, &cond)
+  }.sort
+end
 
 if $extout
   Config.expand(extout = "#$extout", Config::CONFIG.merge("topdir"=>$topdir))
@@ -428,7 +447,8 @@ SRC
   $extpath.delete("$(topdir)")
   $extflags = libpathflag($extpath) << " " << $extflags.strip
   conf = [
-    ['SETUP', $setup], [$enable_shared ? 'DLDOBJS' : 'EXTOBJS', $extobjs],
+    ['SETUP', $setup],
+    [enable_config("shared", $enable_shared) ? 'DLDOBJS' : 'EXTOBJS', $extobjs],
     ['EXTLIBS', $extlibs.join(' ')], ['EXTLDFLAGS', $extflags]
   ].map {|n, v|
     "#{n}=#{v}" if v and !(v = v.strip).empty?
@@ -455,6 +475,9 @@ puts "making #{rubies.join(', ')}"
 $stdout.flush
 $mflags.concat(rubies)
 
+if $nmake == ?b
+  $mflags.collect {|flag| flag.sub!(/\A(?=\w+=)/, "-D")}
+end
 system($make, *sysquote($mflags)) or exit($?.exitstatus)
 
 #Local variables:
