@@ -2,8 +2,8 @@
 
   socket.c -
 
-  $Author: matz $
-  $Date: 2005/03/08 09:56:17 $
+  $Author: akr $
+  $Date: 2005/09/18 18:22:44 $
   created at: Thu Mar 31 12:21:29 JST 1994
 
   Copyright (C) 1993-2001 Yukihiro Matsumoto
@@ -173,6 +173,30 @@ ruby_getaddrinfo(nodename, servname, hints, res)
 #define getaddrinfo(node,serv,hints,res) ruby_getaddrinfo((node),(serv),(hints),(res))
 #endif
 
+#if defined(_AIX)
+static int
+ruby_getaddrinfo__aix(nodename, servname, hints, res)
+     char *nodename;
+     char *servname;
+     struct addrinfo *hints;
+     struct addrinfo **res;
+{
+    int error = getaddrinfo(nodename, servname, hints, res);
+    struct addrinfo *r;
+    if (error)
+	return error;
+    for (r = *res; r != NULL; r = r->ai_next) {
+	if (r->ai_addr->sa_family == 0)
+	    r->ai_addr->sa_family = r->ai_family;
+	if (r->ai_addr->sa_len == 0)
+	    r->ai_addr->sa_len = r->ai_addrlen;
+    }
+    return 0;
+}
+#undef getaddrinfo
+#define getaddrinfo(node,serv,hints,res) ruby_getaddrinfo__aix((node),(serv),(hints),(res))
+#endif
+
 #ifdef HAVE_CLOSESOCKET
 #undef close
 #define close closesocket
@@ -475,11 +499,15 @@ bsock_send(argc, argv, sock)
     rb_thread_fd_writable(fd);
   retry:
     if (!NIL_P(to)) {
+        TRAP_BEG;
 	n = sendto(fd, RSTRING(mesg)->ptr, RSTRING(mesg)->len, NUM2INT(flags),
 		   (struct sockaddr*)RSTRING(to)->ptr, RSTRING(to)->len);
+        TRAP_END;
     }
     else {
+        TRAP_BEG;
 	n = send(fd, RSTRING(mesg)->ptr, RSTRING(mesg)->len, NUM2INT(flags));
+        TRAP_END;
     }
     if (n < 0) {
 	if (rb_io_wait_writable(fd)) {
@@ -851,7 +879,8 @@ static int
 wait_connectable(fd)
     int fd;
 {
-    int sockerr, sockerrlen;
+    int sockerr;
+    socklen_t sockerrlen;
     fd_set fds_w;
     fd_set fds_e;
 
@@ -1677,9 +1706,9 @@ unix_send_io(sock, val)
 
 #if FD_PASSING_BY_MSG_CONTROL
     msg.msg_control = (caddr_t)&cmsg;
-    msg.msg_controllen = sizeof(struct cmsghdr) + sizeof(int);
+    msg.msg_controllen = CMSG_SPACE(sizeof(int));
     msg.msg_flags = 0;
-    cmsg.hdr.cmsg_len = sizeof(struct cmsghdr) + sizeof(int);
+    cmsg.hdr.cmsg_len = CMSG_LEN(sizeof(int));
     cmsg.hdr.cmsg_level = SOL_SOCKET;
     cmsg.hdr.cmsg_type = SCM_RIGHTS;
     cmsg.fd = fd;
@@ -1752,9 +1781,9 @@ unix_recv_io(argc, argv, sock)
 
 #if FD_PASSING_BY_MSG_CONTROL
     msg.msg_control = (caddr_t)&cmsg;
-    msg.msg_controllen = sizeof(struct cmsghdr) + sizeof(int);
+    msg.msg_controllen = CMSG_SPACE(sizeof(int));
     msg.msg_flags = 0;
-    cmsg.hdr.cmsg_len = sizeof(struct cmsghdr) + sizeof(int);
+    cmsg.hdr.cmsg_len = CMSG_LEN(sizeof(int));
     cmsg.hdr.cmsg_level = SOL_SOCKET;
     cmsg.hdr.cmsg_type = SCM_RIGHTS;
     cmsg.fd = -1;
@@ -1767,18 +1796,34 @@ unix_recv_io(argc, argv, sock)
     if (recvmsg(fileno(fptr->f), &msg, 0) == -1)
 	rb_sys_fail("recvmsg(2)");
 
-    if (
 #if FD_PASSING_BY_MSG_CONTROL
-	msg.msg_controllen != sizeof(struct cmsghdr) + sizeof(int) ||
-        cmsg.hdr.cmsg_len != sizeof(struct cmsghdr) + sizeof(int) ||
-	cmsg.hdr.cmsg_level != SOL_SOCKET ||
-	cmsg.hdr.cmsg_type != SCM_RIGHTS
-#else
-        msg.msg_accrightslen != sizeof(fd)
-#endif
-	) {
-	rb_raise(rb_eSocket, "file descriptor was not passed");
+    if (msg.msg_controllen != CMSG_SPACE(sizeof(int))) {
+      rb_raise(rb_eSocket,
+          "file descriptor was not passed (msg_controllen : %d != %d)",
+          msg.msg_controllen, CMSG_SPACE(sizeof(int)));
     }
+    if (cmsg.hdr.cmsg_len != CMSG_SPACE(0) + sizeof(int)) {
+      rb_raise(rb_eSocket,
+          "file descriptor was not passed (cmsg_len : %d != %d)",
+          cmsg.hdr.cmsg_len, CMSG_SPACE(0) + sizeof(int));
+    }
+    if (cmsg.hdr.cmsg_level != SOL_SOCKET) {
+      rb_raise(rb_eSocket,
+          "file descriptor was not passed (cmsg_level : %d != %d)",
+          cmsg.hdr.cmsg_level, SOL_SOCKET);
+    }
+    if (cmsg.hdr.cmsg_type != SCM_RIGHTS) {
+      rb_raise(rb_eSocket,
+          "file descriptor was not passed (cmsg_type : %d != %d)",
+          cmsg.hdr.cmsg_type, SCM_RIGHTS);
+    }
+#else
+    if (msg.msg_accrightslen != sizeof(fd)) {
+	rb_raise(rb_eSocket,
+            "file descriptor was not passed (accrightslen) : %d != %d",
+            msg.msg_accrightslen, sizeof(fd));
+    }
+#endif
 
 #if FD_PASSING_BY_MSG_CONTROL
     fd = cmsg.fd;
@@ -2413,7 +2458,9 @@ sock_s_getnameinfo(argc, argv)
 		 * 4th element holds numeric form, don't resolve.
 		 * see ipaddr().
 		 */
+#ifdef AI_NUMERICHOST /* AIX 4.3.3 doesn't have AI_NUMERICHOST. */
 		hints.ai_flags |= AI_NUMERICHOST;
+#endif
 	    }
 	}
 	else {
