@@ -1,6 +1,6 @@
 #
 #               tk/text.rb - Tk text classes
-#                       $Date: 2005/04/09 09:27:24 $
+#                       $Date: 2005/10/22 22:16:24 $
 #                       by Yukihiro Matsumoto <matz@caelum.co.jp>
 require 'tk'
 require 'tk/itemfont'
@@ -187,6 +187,11 @@ class TkText<TkTextWin
   end
   private :create_self
 
+  def __strval_optkeys
+    super() << 'inactiveseletcionbackground'
+  end
+  private :__strval_optkeys
+
   def self.at(x, y)
     TkText::IndexString.at(x, y)
   end
@@ -250,13 +255,15 @@ class TkText<TkTextWin
   end
 
   def tag_names(index=None)
-    tk_split_simplelist(_fromUTF8(tk_send_without_enc('tag', 'names', _get_eval_enc_str(index)))).collect{|elt|
+    #tk_split_simplelist(_fromUTF8(tk_send_without_enc('tag', 'names', _get_eval_enc_str(index)))).collect{|elt|
+    tk_split_simplelist(tk_send_without_enc('tag', 'names', _get_eval_enc_str(index)), false, true).collect{|elt|
       tagid2obj(elt)
     }
   end
 
   def mark_names
-    tk_split_simplelist(_fromUTF8(tk_send_without_enc('mark', 'names'))).collect{|elt|
+    #tk_split_simplelist(_fromUTF8(tk_send_without_enc('mark', 'names'))).collect{|elt|
+    tk_split_simplelist(tk_send_without_enc('mark', 'names'), false, true).collect{|elt|
       tagid2obj(elt)
     }
   end
@@ -328,15 +335,19 @@ class TkText<TkTextWin
       if slot
         case slot.to_s
         when 'text', 'label', 'show', 'data', 'file'
-          conf = tk_split_simplelist(_fromUTF8(tk_send_without_enc('image', 'configure', _get_eval_enc_str(index), "-#{slot}")))
+          #conf = tk_split_simplelist(_fromUTF8(tk_send_without_enc('image', 'configure', _get_eval_enc_str(index), "-#{slot}")))
+          conf = tk_split_simplelist(tk_send_without_enc('image', 'configure', _get_eval_enc_str(index), "-#{slot}"), false, true)
         else
-          conf = tk_split_list(_fromUTF8(tk_send_without_enc('image', 'configure', _get_eval_enc_str(index), "-#{slot}")))
+          #conf = tk_split_list(_fromUTF8(tk_send_without_enc('image', 'configure', _get_eval_enc_str(index), "-#{slot}")))
+          conf = tk_split_list(tk_send_without_enc('image', 'configure', _get_eval_enc_str(index), "-#{slot}"), 0, false, true)
         end
         conf[0] = conf[0][1..-1]
         conf
       else
-        tk_split_simplelist(_fromUTF8(tk_send_without_enc('image', 'configure', _get_eval_enc_str(index)))).collect{|conflist|
-          conf = tk_split_simplelist(conflist)
+        # tk_split_simplelist(_fromUTF8(tk_send_without_enc('image', 'configure', _get_eval_enc_str(index)))).collect{|conflist|
+        #  conf = tk_split_simplelist(conflist)
+        tk_split_simplelist(tk_send_without_enc('image', 'configure', _get_eval_enc_str(index)), false, false).collect{|conflist|
+          conf = tk_split_simplelist(conflist, false, true)
           conf[0] = conf[0][1..-1]
           case conf[0]
           when 'text', 'label', 'show', 'data', 'file'
@@ -364,16 +375,20 @@ class TkText<TkTextWin
       if slot
         case slot.to_s
         when 'text', 'label', 'show', 'data', 'file'
-          conf = tk_split_simplelist(_fromUTF8(tk_send_without_enc('image', 'configure', _get_eval_enc_str(index), "-#{slot}")))
+          #conf = tk_split_simplelist(_fromUTF8(tk_send_without_enc('image', 'configure', _get_eval_enc_str(index), "-#{slot}")))
+          conf = tk_split_simplelist(tk_send_without_enc('image', 'configure', _get_eval_enc_str(index), "-#{slot}"), false, true)
         else
-          conf = tk_split_list(_fromUTF8(tk_send_without_enc('image', 'configure', _get_eval_enc_str(index), "-#{slot}")))
+          #conf = tk_split_list(_fromUTF8(tk_send_without_enc('image', 'configure', _get_eval_enc_str(index), "-#{slot}")))
+          conf = tk_split_list(tk_send_without_enc('image', 'configure', _get_eval_enc_str(index), "-#{slot}"), 0, false, true)
         end
         key = conf.shift[1..-1]
         { key => conf }
       else
         ret = {}
-        tk_split_simplelist(_fromUTF8(tk_send_without_enc('image', 'configure', _get_eval_enc_str(index)))).each{|conflist|
-          conf = tk_split_simplelist(conflist)
+        #tk_split_simplelist(_fromUTF8(tk_send_without_enc('image', 'configure', _get_eval_enc_str(index)))).each{|conflist|
+        #  conf = tk_split_simplelist(conflist)
+        tk_split_simplelist(tk_send_without_enc('image', 'configure', _get_eval_enc_str(index)), false, false).each{|conflist|
+          conf = tk_split_simplelist(conflist, false, true)
           key = conf.shift[1..-1]
           case key
           when 'text', 'label', 'show', 'data', 'file'
@@ -426,7 +441,8 @@ class TkText<TkTextWin
   end
 
   def image_names
-    tk_split_simplelist(_fromUTF8(tk_send_without_enc('image', 'names'))).collect{|elt|
+    #tk_split_simplelist(_fromUTF8(tk_send_without_enc('image', 'names'))).collect{|elt|
+    tk_split_simplelist(tk_send_without_enc('image', 'names'), false, true).collect{|elt|
       tagid2obj(elt)
     }
   end
@@ -449,13 +465,13 @@ class TkText<TkTextWin
         args << tags.shift.collect{|x|_get_eval_string(x)}.join(' ')  # taglist
         args << tags.shift if tags.size > 0                           # chars
       end
-      super index, *args
+      super(index, *args)
     else
       # single chars-taglist argument :: str, tag, tag, ...
       if tags.size == 0
-        super index, chars
+        super(index, chars)
       else
-        super index, chars, tags.collect{|x|_get_eval_string(x)}.join(' ')
+        super(index, chars, tags.collect{|x|_get_eval_string(x)}.join(' '))
       end
     end
   end
@@ -465,7 +481,7 @@ class TkText<TkTextWin
     @tags.each_value do |t|
       t.destroy
     end
-    super
+    super()
   end
 
   def backspace
@@ -853,8 +869,11 @@ class TkText<TkTextWin
   end
 
   def tag_ranges(tag)
+    #l = tk_split_simplelist(tk_send_without_enc('tag', 'ranges',
+    #                                            _get_eval_enc_str(tag)))
     l = tk_split_simplelist(tk_send_without_enc('tag', 'ranges',
-                                                _get_eval_enc_str(tag)))
+                                                _get_eval_enc_str(tag)), 
+                            false, true)
     r = []
     while key=l.shift
       r.push [TkText::IndexString.new(key), TkText::IndexString.new(l.shift)]
@@ -1065,7 +1084,8 @@ class TkText<TkTextWin
 =end
 
   def window_names
-    tk_split_simplelist(_fromUTF8(tk_send_without_enc('window', 'names'))).collect{|elt|
+    # tk_split_simplelist(_fromUTF8(tk_send_without_enc('window', 'names'))).collect{|elt|
+    tk_split_simplelist(tk_send_without_enc('window', 'names'), false, true).collect{|elt|
       tagid2obj(elt)
     }
   end
