@@ -2,8 +2,8 @@
 
   intern.h -
 
-  $Author: ocean $
-  $Date: 2004/07/15 11:42:50 $
+  $Author: nobu $
+  $Date: 2005/10/22 04:09:23 $
   created at: Thu Jun 10 14:22:17 JST 1993
 
   Copyright (C) 1993-2003 Yukihiro Matsumoto
@@ -136,8 +136,8 @@ NORETURN(void rb_name_error __((ID, const char*, ...)));
 NORETURN(void rb_invalid_str _((const char*, const char*)));
 void rb_compile_error __((const char*, ...));
 void rb_compile_error_append __((const char*, ...));
-NORETURN(void rb_load_fail _((char*)));
-NORETURN(void rb_error_frozen _((char*)));
+NORETURN(void rb_load_fail _((const char*)));
+NORETURN(void rb_error_frozen _((const char*)));
 void rb_check_frozen _((VALUE));
 /* eval.c */
 RUBY_EXTERN struct RNode *ruby_current_node;
@@ -164,6 +164,7 @@ void rb_dvar_asgn _((ID, VALUE));
 void rb_dvar_push _((ID, VALUE));
 VALUE *rb_svar _((int));
 VALUE rb_eval_cmd _((VALUE, VALUE, int));
+int rb_obj_respond_to _((VALUE, ID, int));
 int rb_respond_to _((VALUE, ID));
 void rb_interrupt _((void));
 VALUE rb_apply _((VALUE, ID, VALUE));
@@ -210,6 +211,7 @@ VALUE rb_thread_create _((VALUE (*)(ANYARGS), void*));
 void rb_thread_interrupt _((void));
 void rb_thread_trap_eval _((VALUE, int, int));
 void rb_thread_signal_raise _((char*));
+void rb_thread_signal_exit _((void));
 int rb_thread_select _((int, fd_set *, fd_set *, fd_set *, struct timeval *));
 void rb_thread_wait_for _((struct timeval));
 VALUE rb_thread_current _((void));
@@ -217,6 +219,7 @@ VALUE rb_thread_main _((void));
 VALUE rb_thread_local_aref _((VALUE, ID));
 VALUE rb_thread_local_aset _((VALUE, ID, VALUE));
 void rb_thread_atfork _((void));
+VALUE rb_funcall_rescue __((VALUE, ID, int, ...));
 /* file.c */
 int eaccess _((const char*, int));
 VALUE rb_file_s_expand_path _((int, VALUE *));
@@ -240,11 +243,14 @@ void rb_gc_mark _((VALUE));
 void rb_gc_force_recycle _((VALUE));
 void rb_gc _((void));
 void rb_gc_copy_finalizer _((VALUE,VALUE));
+void rb_gc_finalize_deferred _((void));
 void rb_gc_call_finalizer_at_exit _((void));
 VALUE rb_gc_enable _((void));
 VALUE rb_gc_disable _((void));
 VALUE rb_gc_start _((void));
 /* hash.c */
+void st_foreach_safe _((struct st_table *, int (*)(ANYARGS), unsigned long));
+void rb_hash_foreach _((VALUE, int (*)(ANYARGS), VALUE));
 VALUE rb_hash _((VALUE));
 VALUE rb_hash_new _((void));
 VALUE rb_hash_freeze _((VALUE));
@@ -305,6 +311,7 @@ VALUE rb_obj_freeze _((VALUE));
 VALUE rb_obj_id _((VALUE));
 VALUE rb_obj_class _((VALUE));
 VALUE rb_class_real _((VALUE));
+VALUE rb_class_inherited_p _((VALUE, VALUE));
 VALUE rb_convert_type _((VALUE,int,const char*,const char*));
 VALUE rb_check_convert_type _((VALUE,int,const char*,const char*));
 VALUE rb_to_int _((VALUE));
@@ -328,6 +335,7 @@ int rb_is_instance_id _((ID));
 int rb_is_class_id _((ID));
 int rb_is_local_id _((ID));
 int rb_is_junk_id _((ID));
+int rb_symname_p _((const char*));
 VALUE rb_backref_get _((void));
 void rb_backref_set _((VALUE));
 VALUE rb_lastline_get _((void));
@@ -363,8 +371,8 @@ const char* rb_get_kcode _((void));
 /* ruby.c */
 RUBY_EXTERN VALUE rb_argv;
 RUBY_EXTERN VALUE rb_argv0;
-void rb_load_file _((char*));
-void ruby_script _((char*));
+void rb_load_file _((const char*));
+void ruby_script _((const char*));
 void ruby_prog_init _((void));
 void ruby_set_argv _((int, char**));
 void ruby_process_options _((int, char**));
@@ -399,6 +407,8 @@ VALUE rb_str_buf_cat2 _((VALUE, const char*));
 VALUE rb_obj_as_string _((VALUE));
 VALUE rb_check_string_type _((VALUE));
 VALUE rb_str_dup _((VALUE));
+VALUE rb_str_locktmp _((VALUE));
+VALUE rb_str_unlocktmp _((VALUE));
 VALUE rb_str_dup_frozen _((VALUE));
 VALUE rb_str_plus _((VALUE, VALUE));
 VALUE rb_str_times _((VALUE, VALUE));
@@ -429,6 +439,8 @@ VALUE rb_struct_aref _((VALUE, VALUE));
 VALUE rb_struct_aset _((VALUE, VALUE, VALUE));
 VALUE rb_struct_getmember _((VALUE, ID));
 VALUE rb_struct_iv_get _((VALUE, char*));
+VALUE rb_struct_s_members _((VALUE));
+VALUE rb_struct_members _((VALUE));
 /* time.c */
 VALUE rb_time_new _((time_t, time_t));
 /* variable.c */
@@ -439,7 +451,7 @@ VALUE rb_path2class _((const char*));
 void rb_name_class _((VALUE, ID));
 VALUE rb_class_name _((VALUE));
 void rb_autoload _((VALUE, ID, const char*));
-void rb_autoload_load _((VALUE, ID));
+VALUE rb_autoload_load _((VALUE, ID));
 VALUE rb_autoload_p _((VALUE, ID));
 void rb_gc_mark_global_tbl _((void));
 VALUE rb_f_trace_var _((int, VALUE*));
