@@ -215,7 +215,6 @@ module Logging
 end
 
 def xsystem command
-  Config.expand(command)
   Logging::open do
     puts command.quote
     system(command)
@@ -223,7 +222,6 @@ def xsystem command
 end
 
 def xpopen command, *mode, &block
-  Config.expand(command)
   Logging::open do
     case mode[0]
     when nil, /^r/
@@ -262,31 +260,33 @@ end
 
 def link_command(ldflags, opt="", libpath=$LIBPATH)
   Config::expand(TRY_LINK.dup,
-		 'hdrdir' => $hdrdir,
-		 'src' => CONFTEST_C,
-		 'INCFLAGS' => $INCFLAGS,
-		 'CPPFLAGS' => $CPPFLAGS,
-		 'CFLAGS' => "#$CFLAGS",
-		 'ARCH_FLAG' => "#$ARCH_FLAG",
-		 'LDFLAGS' => "#$LDFLAGS #{ldflags}",
-		 'LIBPATH' => libpathflag(libpath),
-		 'LOCAL_LIBS' => "#$LOCAL_LIBS #$libs",
-		 'LIBS' => "#$LIBRUBYARG_STATIC #{opt} #$LIBS")
+                 CONFIG.merge('hdrdir' => $hdrdir.quote,
+                              'src' => CONFTEST_C,
+                              'INCFLAGS' => $INCFLAGS,
+                              'CPPFLAGS' => $CPPFLAGS,
+                              'CFLAGS' => "#$CFLAGS",
+                              'ARCH_FLAG' => "#$ARCH_FLAG",
+                              'LDFLAGS' => "#$LDFLAGS #{ldflags}",
+                              'LIBPATH' => libpathflag(libpath),
+                              'LOCAL_LIBS' => "#$LOCAL_LIBS #$libs",
+                              'LIBS' => "#$LIBRUBYARG_STATIC #{opt} #$LIBS"))
 end
 
 def cc_command(opt="")
-  "$(CC) -c #$INCFLAGS -I#{$hdrdir} " \
-  "#$CPPFLAGS #$CFLAGS #$ARCH_FLAG #{opt} #{CONFTEST_C}"
+  Config::expand("$(CC) -c #$INCFLAGS -I$(hdrdir) " \
+                 "#$CPPFLAGS #$CFLAGS #$ARCH_FLAG #{opt} #{CONFTEST_C}",
+		 CONFIG.merge('hdrdir' => $hdrdir.quote))
 end
 
 def cpp_command(outfile, opt="")
-  "$(CPP) #$INCFLAGS -I#{$hdrdir} " \
-  "#$CPPFLAGS #$CFLAGS #{opt} #{CONFTEST_C} #{outfile}"
+  Config::expand("$(CPP) #$INCFLAGS -I$(hdrdir) " \
+                 "#$CPPFLAGS #$CFLAGS #{opt} #{CONFTEST_C} #{outfile}",
+		 CONFIG.merge('hdrdir' => $hdrdir.quote))
 end
 
 def libpathflag(libpath=$LIBPATH)
   libpath.map{|x|
-    (x == "$(topdir)" ? LIBPATHFLAG : LIBPATHFLAG+RPATHFLAG) % x
+    (x == "$(topdir)" ? LIBPATHFLAG : LIBPATHFLAG+RPATHFLAG) % x.quote
   }.join
 end
 
@@ -706,9 +706,73 @@ def check_sizeof(type, header = nil, &b)
   size
 end
 
+def scalar_ptr_type?(type, member = nil, headers = nil, &b)
+  try_compile(<<"SRC", &b)   # pointer
+#{COMMON_HEADERS}
+#{cpp_include(headers)}
+/*top*/
+volatile #{type} conftestval;
+int main() { return 0; }
+int t() {return (int)(1-*(conftestval#{member ? ".#{member}" : ""}));}
+SRC
+end
+
+def scalar_type?(type, member = nil, headers = nil, &b)
+  try_compile(<<"SRC", &b)   # pointer
+#{COMMON_HEADERS}
+#{cpp_include(headers)}
+/*top*/
+volatile #{type} conftestval;
+int main() { return 0; }
+int t() {return (int)(1-(conftestval#{member ? ".#{member}" : ""}));}
+SRC
+end
+
+def what_type?(type, member = nil, headers = nil, &b)
+  m = "#{type}"
+  name = type
+  if member
+    m << "." << member
+    name = "(((#{type} *)0)->#{member})"
+  end
+  m << " in #{headers.inspect}" if headers
+  fmt = "seems %s"
+  def fmt.%(x)
+    x ? super : "unknown"
+  end
+  checking_for m, fmt do
+    if scalar_ptr_type?(type, member, headers, &b)
+      if try_static_assert("sizeof(*#{name}) == 1", headers)
+        "string"
+      end
+    elsif scalar_type?(type, member, headers, &b)
+      if try_static_assert("sizeof(#{name}) > sizeof(long)", headers)
+        "long long"
+      elsif try_static_assert("sizeof(#{name}) > sizeof(int)", headers)
+        "long"
+      elsif try_static_assert("sizeof(#{name}) > sizeof(short)", headers)
+        "int"
+      elsif try_static_assert("sizeof(#{name}) > 1", headers)
+        "short"
+      else
+        "char"
+      end
+    end
+  end
+end
+
 def find_executable0(bin, path = nil)
-  path = (path || ENV['PATH']).split(File::PATH_SEPARATOR)
   ext = config_string('EXEEXT')
+  if File.expand_path(bin) == bin
+    return bin if File.executable?(bin)
+    return file if ext and File.executable?(file = bin + ext)
+    return nil
+  end
+  if path ||= ENV['PATH']
+    path = path.split(File::PATH_SEPARATOR)
+  else
+    path = %w[/usr/local/bin /usr/ucb /usr/bin /bin]
+  end
   file = nil
   path.each do |dir|
     return file if File.executable?(file = File.join(dir, bin))
@@ -729,11 +793,25 @@ def arg_config(config, *defaults, &block)
   $configure_args.fetch(config.tr('_', '-'), *defaults, &block)
 end
 
-def with_config(config, *defaults, &block)
-  unless /^--with[-_]/ =~ config
-    config = '--with-' + config
+def with_config(config, *defaults)
+  config = config.sub(/^--with[-_]/, '')
+  val = arg_config("--with-"+config) do
+    if arg_config("--without-"+config)
+      false
+    elsif block_given?
+      yield(config, *defaults)
+    else
+      break *defaults
+    end
+  end
+  case val
+  when "yes"
+    true
+  when "no"
+    false
+  else
+    val
   end
-  arg_config(config, *defaults, &block)
 end
 
 def enable_config(config, *defaults)
@@ -834,20 +912,24 @@ end
 def configuration(srcdir)
   mk = []
   vpath = %w[$(srcdir) $(topdir) $(hdrdir)]
-  if !CROSS_COMPILING && CONFIG['build_os'] == 'cygwin' && CONFIG['target_os'] != 'cygwin'
-    vpath.each {|p| p.sub!(/.*/, '$(shell cygpath -u \&)')}
-  end
-  if !CROSS_COMPILING && CONFIG['build_os'] == 'msdosdjgpp'
-    CONFIG['PATH_SEPARATOR'] = ';'
+  if !CROSS_COMPILING
+    case CONFIG['build_os']
+    when 'cygwin'
+      if CONFIG['target_os'] != 'cygwin'
+        vpath.each {|p| p.sub!(/.*/, '$(shell cygpath -u \&)')}
+      end
+    when 'msdosdjgpp', 'mingw32'
+      CONFIG['PATH_SEPARATOR'] = ';'
+    end
   end
   mk << %{
 SHELL = /bin/sh
 
 #### Start of system configuration section. ####
 
-srcdir = #{srcdir.gsub(/\$\((srcdir)\)|\$\{(srcdir)\}/) {CONFIG[$1||$2]}}
-topdir = #{$extmk ? CONFIG["topdir"] : $topdir}
-hdrdir = #{$extmk ? CONFIG["hdrdir"] : '$(topdir)'}
+srcdir = #{srcdir.gsub(/\$\((srcdir)\)|\$\{(srcdir)\}/) {CONFIG[$1||$2]}.quote}
+topdir = #{($extmk ? CONFIG["topdir"] : $topdir).quote}
+hdrdir = #{$extmk ? CONFIG["hdrdir"].quote : '$(topdir)'}
 VPATH = #{vpath.join(CONFIG['PATH_SEPARATOR'])}
 }
   drive = File::PATH_SEPARATOR == ';' ? /\A\w:/ : /\A/
@@ -863,6 +945,12 @@ VPATH = #{vpath.join(CONFIG['PATH_SEPARATOR'])}
     next unless /^(?:src|top|hdr|(.*))dir$/ =~ key and $1
     mk << "#{key} = #{with_destdir(var.sub(drive, ''))}\n"
   end
+  if !$extmk and !$configure_args.has_key?('--ruby') and
+      sep = config_string('BUILD_FILE_SEPARATOR')
+    sep = ":/=#{sep}"
+  else
+    sep = ""
+  end
   mk << %{
 CC = #{CONFIG['CC']}
 LIBRUBY = #{CONFIG['LIBRUBY']}
@@ -884,12 +972,12 @@ arch = #{CONFIG['arch']}
 sitearch = #{CONFIG['sitearch']}
 ruby_version = #{Config::CONFIG['ruby_version']}
 ruby = #{$ruby}
-RUBY = #{($nmake && !$extmk && !$configure_args.has_key?('--ruby')) ? '$(ruby:/=\)' : '$(ruby)'}
+RUBY = $(ruby#{sep})
 RM = #{config_string('RM') || '$(RUBY) -run -e rm -- -f'}
 MAKEDIRS = #{config_string('MAKEDIRS') || '@$(RUBY) -run -e mkdir -- -p'}
 INSTALL = #{config_string('INSTALL') || '@$(RUBY) -run -e install -- -vp'}
-INSTALL_PROG = $(INSTALL) -m 0755
-INSTALL_DATA = $(INSTALL) -m 0644
+INSTALL_PROG = #{config_string('INSTALL_PROG') || '$(INSTALL) -m 0755'}
+INSTALL_DATA = #{config_string('INSTALL_DATA') || '$(INSTALL) -m 0644'}
 COPY = #{config_string('CP') || '@$(RUBY) -run -e cp -- -v'}
 
 #### End of system configuration section. ####
@@ -1019,13 +1107,23 @@ static:		$(STATIC_LIB)#{$extout ? " install-rb" : ""}
     dest = "#{dir}/#{f}"
     mfile.print "install-so: #{dest}\n"
     unless $extout
-      mfile.print "#{dest}: #{f}\n\t$(INSTALL_PROG) #{f} #{dir}\n"
+      mfile.print "#{dest}: #{f}\n"
+      if (sep = config_string('BUILD_FILE_SEPARATOR'))
+        f.gsub!("/", sep)
+        dir.gsub!("/", sep)
+        sep = ":/="+sep
+        f.gsub!(/(\$\(\w+)(\))/) {$1+sep+$2}
+        f.gsub!(/(\$\{\w+)(\})/) {$1+sep+$2}
+        dir.gsub!(/(\$\(\w+)(\))/) {$1+sep+$2}
+        dir.gsub!(/(\$\{\w+)(\})/) {$1+sep+$2}
+      end
+      mfile.print "\t$(INSTALL_PROG) #{f} #{dir}\n"
     end
   end
-  dirs << (dir = "$(RUBYLIBDIR)")
   mfile.print("install-rb: pre-install-rb install-rb-default\n")
   mfile.print("install-rb-default: pre-install-rb-default\n")
-  mfile.print("pre-install-rb pre-install-rb-default: #{dir}\n")
+  mfile.print("pre-install-rb: Makefile\n")
+  mfile.print("pre-install-rb-default: Makefile\n")
   for sfx, i in [["-default", [["lib/**/*.rb", "$(RUBYLIBDIR)", "lib"]]], ["", $INSTALLFILES]]
     files = install_files(mfile, i, nil, srcprefix) or next
     for dir, *files in files
@@ -1100,11 +1198,13 @@ site-install-rb: install-rb
 
   depend = File.join(srcdir, "depend")
   if File.exist?(depend)
+    suffixes = []
+    depout = []
     open(depend, "r") do |dfile|
       mfile.printf "###\n"
       cont = implicit = nil
       impconv = proc do
-	COMPILE_RULES.each {|rule| mfile.print(rule % implicit[0], implicit[1])}
+	COMPILE_RULES.each {|rule| depout << (rule % implicit[0]) << implicit[1]}
 	implicit = nil
       end
       ruleconv = proc do |line|
@@ -1117,12 +1217,13 @@ site-install-rb: install-rb
 	  end
 	end
 	if m = /\A\.(\w+)\.(\w+)(?:\s*:)/.match(line)
+	  suffixes << m[1] << m[2]
 	  implicit = [[m[1], m[2]], [m.post_match]]
 	  next
 	elsif RULE_SUBST and /\A[$\w][^#]*:/ =~ line
 	  line.gsub!(%r"(\s)(?!\.)([^$(){}+=:\s\/\\,]+)(?=\s|\z)") {$1 + RULE_SUBST % $2}
 	end
-	mfile.print line
+	depout << line
       end
       while line = dfile.gets()
 	line.gsub!(/\.o\b/, ".#{$OBJEXT}")
@@ -1142,6 +1243,10 @@ site-install-rb: install-rb
 	impconv.call
       end
     end
+    unless suffixes.empty?
+      mfile.print ".SUFFIXES: .", suffixes.uniq.join(" ."), "\n\n"
+    end
+    mfile.print depout
   else
     headers = %w[ruby.h defines.h]
     if RULE_SUBST
@@ -1225,8 +1330,11 @@ end
 Config::CONFIG["srcdir"] = CONFIG["srcdir"] =
   $srcdir = arg_config("--srcdir", File.dirname($0))
 $configure_args["--topsrcdir"] ||= $srcdir
-$curdir = arg_config("--curdir", Dir.pwd)
-Config.expand(curdir = $curdir.dup)
+if $curdir = arg_config("--curdir")
+  Config.expand(curdir = $curdir.dup)
+else
+  curdir = $curdir = "."
+end
 unless File.expand_path(Config::CONFIG["topdir"]) == File.expand_path(curdir)
   CONFIG["topdir"] = $curdir
   Config::CONFIG["topdir"] = curdir
