@@ -192,7 +192,7 @@
 #
 class OptionParser
   # :stopdoc:
-  RCSID = %w$Id: optparse.rb,v 1.40.2.2 2004/07/15 14:54:01 nobu Exp $[1..-1].each {|s| s.freeze}.freeze
+  RCSID = %w$Id: optparse.rb,v 1.40.2.7 2005/09/26 12:44:47 nobu Exp $[1..-1].each {|s| s.freeze}.freeze
   Version = (RCSID[1].split('.').collect {|s| s.to_i}.extend(Comparable).freeze if RCSID[1])
   LastModified = (Time.gm(*RCSID[2, 2].join('-').scan(/\d+/).collect {|s| s.to_i}) if RCSID[2])
   Release = RCSID[2]
@@ -207,9 +207,9 @@ class OptionParser
   # and resolved against a list of acceptable values.
   #
   module Completion
-    def complete(key, pat = nil)
+    def complete(key, icase = false, pat = nil)
       pat ||= Regexp.new('\A' + Regexp.quote(key).gsub(/\w+\b/, '\&\w*'),
-                         ignore_case?)
+                         icase)
       canon, sw, k, v, cn = nil
       candidates = []
       each do |k, *v|
@@ -250,10 +250,6 @@ class OptionParser
     def convert(opt = nil, val = nil, *)
       val
     end
-
-    def ignore_case?
-      false
-    end
   end
 
 
@@ -263,11 +259,6 @@ class OptionParser
   class OptionMap < Hash
     include Completion
   end
-  class OptionCaseMap < OptionMap
-    def ignore_case?
-      true
-    end
-  end
 
 
   #
@@ -450,12 +441,12 @@ class OptionParser
       #
       # Raises an exception if argument is not present.
       #
-      def parse(arg, argv, &error)
+      def parse(arg, argv)
         unless arg
           raise MissingArgument if argv.empty?
           arg = argv.shift
         end
-        conv_arg(*parse_arg(arg, &error))
+        conv_arg(*parse_arg(arg) {|*exc| raise(*exc)})
       end
     end
 
@@ -476,11 +467,11 @@ class OptionParser
     end
 
     #
-    # ?
+    # Switch that takes an argument, which does not begin with '-'.
     #
     class PlacedArgument < self
       #
-      # ?
+      # Returns nil if argument is not present or begins with '-'.
       #
       def parse(arg, argv, &error)
         if !(val = arg) and (argv.empty? or /\A-/ =~ (val = argv[0]))
@@ -523,7 +514,7 @@ class OptionParser
     def initialize
       @atype = {}
       @short = OptionMap.new
-      @long = OptionCaseMap.new
+      @long = OptionMap.new
       @list = []
     end
 
@@ -632,13 +623,15 @@ class OptionParser
     #      searching list.
     #    : ((|opt|))
     #      searching key.
+    #    : ((|icase|))
+    #      search case insensitive if true.
     #    : ((|*pat|))
     #      optional pattern for completion.
     #    : (({block}))
     #      yielded with the found value when succeeded.
     #
-    def complete(id, opt, *pat, &block)
-      __send__(id).complete(opt, *pat, &block)
+    def complete(id, opt, icase = false, *pat, &block)
+      __send__(id).complete(opt, icase, *pat, &block)
     end
 
     #
@@ -718,30 +711,38 @@ class OptionParser
   DefaultList.long[''] = Switch::NoArgument.new {throw :terminate}
 
   #
-  # Default options, which never appear in option summary.
+  # OptionParser::Officious
+  # Default options for ARGV, which never appear in option summary.
+  #
+  Officious = {}
+
   #   --help
   #   Shows option summary.
+  Officious['help'] = proc do |parser|
+    Switch::NoArgument.new do
+      puts parser.help
+      exit
+    end
+  end
+
   #   --version
   #   Shows version string if (({::Version})) is defined.
-  #
-  DefaultList.long['help'] = Switch::NoArgument.new do
-    puts ARGV.options
-    exit
-  end
-  DefaultList.long['version'] = Switch::OptionalArgument.new do |pkg|
-    if pkg
-      begin
-        require 'optparse/version'
-      rescue LoadError
-        pkg = nil
-      else
-        show_version(*pkg.split(/,/))
+  Officious['version'] = proc do |parser|
+    Switch::OptionalArgument.new do |pkg|
+      if pkg
+        begin
+          require 'optparse/version'
+        rescue LoadError
+        else
+          show_version(*pkg.split(/,/)) or
+            abort("#{parser.program_name}: no version found in package #{pkg}")
+          exit
+        end
       end
+      v = parser.ver or abort("#{parser.program_name}: version unknown")
+      puts v
+      exit
     end
-    unless pkg
-      v = ARGV.options.ver and puts v
-    end
-    exit
   end
 
   # :startdoc:
@@ -809,9 +810,19 @@ class OptionParser
     @banner = banner
     @summary_width = width
     @summary_indent = indent
+    @default_argv = ARGV
+    add_officious
     yield self if block_given?
   end
 
+  # :nodoc:
+  def add_officious
+    list = base()
+    Officious.each_pair do |opt, block|
+      list.long[opt] ||= block.call(self)
+    end
+  end
+
 =begin
 --- OptionParser.terminate([arg])
     Terminates option parsing. Optional parameter ((|arg|)) would be
@@ -875,9 +886,13 @@ class OptionParser
 --- OptionParser#program_name=(name)
     Program name to be emitted in error message and default banner,
     defaulted to (({$0})).
+--- OptionParser#default_argv
+--- OptionParser#default_argv=(argv)
+    Strings to be parsed in default.
 =end #'#"#`#
   attr_writer :banner, :program_name
   attr_accessor :summary_width, :summary_indent
+  attr_accessor :default_argv
 
   def banner
     @banner ||= "Usage: #{program_name} [options]"
@@ -907,7 +922,7 @@ class OptionParser
   attr_writer :version, :release
 
   def version
-    @version || (defined?(::Version) && ::Version) || (defined?(::VERSION) && ::VERSION)
+    @version || (defined?(::Version) && ::Version)
   end
 
   def release
@@ -1250,7 +1265,7 @@ class OptionParser
     order!(argv, &block)
   end
 
-  def order!(argv = ARGV, &nonopt)
+  def order!(argv = default_argv, &nonopt)
     opt, arg, sw, val, rest = nil
     nonopt ||= proc {|arg| throw :terminate, arg}
     argv.unshift(arg) if arg = catch(:terminate) {
@@ -1260,7 +1275,7 @@ class OptionParser
         when /\A--([^=]*)(?:=(.*))?/
           opt, rest = $1, $2
           begin
-            sw, = complete(:long, opt)
+            sw, = complete(:long, opt, true)
           rescue ParseError
             raise $!.set_option(arg, true)
           end
@@ -1330,7 +1345,7 @@ class OptionParser
     permute!(argv)
   end
 
-  def permute!(argv = ARGV)
+  def permute!(argv = default_argv)
     nonopts = []
     arg = nil
     order!(argv) {|arg| nonopts << arg}
@@ -1356,7 +1371,7 @@ class OptionParser
     parse!(argv)
   end
 
-  def parse!(argv = ARGV)
+  def parse!(argv = default_argv)
     if ENV.include?('POSIXLY_CORRECT')
       order!(argv)
     else
@@ -1414,17 +1429,19 @@ class OptionParser
         searching table.
       : ((|opt|))
         searching key.
+      : ((|icase|))
+        search case insensitive if true.
       : ((|*pat|))
         optional pattern for completion.
       : (({block}))
         yielded with the found value when succeeded.
 =end #'#"#`#
-  def complete(typ, opt, *pat)
+  def complete(typ, opt, icase = false, *pat)
     if pat.empty?
       search(typ, opt) {|sw| return [sw, opt]} # exact match or...
     end
     raise AmbiguousOption, catch(:ambiguous) {
-      visit(:complete, typ, opt, *pat) {|opt, *sw| return sw}
+      visit(:complete, typ, opt, icase, *pat) {|opt, *sw| return sw}
       raise InvalidOption, opt
     }
   end
@@ -1562,6 +1579,21 @@ class OptionParser
     s
   end
 
+=begin
+: Regexp
+  Regular expression with option.
+=end
+  accept(Regexp, %r"\A/((?:\\.|[^\\])*)/([[:alpha:]]+)?\z|.*") do |all, s, o|
+    f = 0
+    if o
+      f |= Regexp::IGNORECASE if /i/ =~ o
+      f |= Regexp::MULTILINE if /m/ =~ o
+      f |= Regexp::EXTENDED if /x/ =~ o
+      k = o.delete("^imx")
+    end
+    Regexp.new(s || all, f, k)
+  end
+
 
 =begin
 = Exceptions
@@ -1646,7 +1678,7 @@ argument.
 ((<OptionParser::ParseError>))
 =end #'#"#`#
   class NeedlessArgument < ParseError
-    const_set(:Reason, 'needles argument'.freeze)
+    const_set(:Reason, 'needless argument'.freeze)
   end
 
 =begin
@@ -1733,6 +1765,7 @@ Extends command line arguments array to parse itself.
 =end #'#"#`#
     def options
       @optparse ||= OptionParser.new
+      @optparse.default_argv = self
       block_given? or return @optparse
       begin
         yield @optparse
