@@ -2,8 +2,8 @@
 
   numeric.c -
 
-  $Author: ocean $
-  $Date: 2005/05/10 01:40:49 $
+  $Author: nobu $
+  $Date: 2005/09/28 14:42:46 $
   created at: Fri Aug 13 18:33:09 JST 1993
 
   Copyright (C) 1993-2003 Yukihiro Matsumoto
@@ -198,6 +198,7 @@ num_sadded(x, name)
     return Qnil;		/* not reached */
 }
 
+/* :nodoc: */
 static VALUE
 num_init_copy(x, y)
     VALUE x, y;
@@ -832,12 +833,13 @@ flo_eq(x, y)
 	break;
       case T_FLOAT:
 	b = RFLOAT(y)->value;
+	if (isnan(b)) return Qfalse;
 	break;
       default:
 	return num_equal(x, y);
     }
     a = RFLOAT(x)->value;
-    if (isnan(a) || isnan(b)) return Qfalse;
+    if (isnan(a)) return Qfalse;
     return (a == b)?Qtrue:Qfalse;
 }
 
@@ -937,12 +939,13 @@ flo_gt(x, y)
 
       case T_FLOAT:
 	b = RFLOAT(y)->value;
+	if (isnan(b)) return Qfalse;
 	break;
 
       default:
 	return rb_num_coerce_relop(x, y);
     }
-    if (isnan(a) || isnan(b)) return Qfalse;
+    if (isnan(a)) return Qfalse;
     return (a > b)?Qtrue:Qfalse;
 }
 
@@ -972,12 +975,13 @@ flo_ge(x, y)
 
       case T_FLOAT:
 	b = RFLOAT(y)->value;
+	if (isnan(b)) return Qfalse;
 	break;
 
       default:
 	return rb_num_coerce_relop(x, y);
     }
-    if (isnan(a) || isnan(b)) return Qfalse;
+    if (isnan(a)) return Qfalse;
     return (a >= b)?Qtrue:Qfalse;
 }
 
@@ -1006,12 +1010,13 @@ flo_lt(x, y)
 
       case T_FLOAT:
 	b = RFLOAT(y)->value;
+	if (isnan(b)) return Qfalse;
 	break;
 
       default:
 	return rb_num_coerce_relop(x, y);
     }
-    if (isnan(a) || isnan(b)) return Qfalse;
+    if (isnan(a)) return Qfalse;
     return (a < b)?Qtrue:Qfalse;
 }
 
@@ -1041,12 +1046,13 @@ flo_le(x, y)
 
       case T_FLOAT:
 	b = RFLOAT(y)->value;
+	if (isnan(b)) return Qfalse;
 	break;
 
       default:
 	return rb_num_coerce_relop(x, y);
     }
-    if (isnan(a) || isnan(b)) return Qfalse;
+    if (isnan(a)) return Qfalse;
     return (a <= b)?Qtrue:Qfalse;
 }
 
@@ -2196,6 +2202,9 @@ fix_pow(x, y)
 	    return rb_big_pow(rb_int2big(a), y);
 	}
 	return rb_float_new(pow((double)a, (double)b));
+    } else if (TYPE(y) == T_FLOAT) {
+        long a = FIX2LONG(x);
+        return rb_float_new(pow((double)a, RFLOAT(y)->value));
     }
     return rb_num_coerce_bin(x, y);
 }
