@@ -2,7 +2,7 @@
 
   re.c -
 
-  $Author: matz $
+  $Author: nobu $
   created at: Mon Aug  9 18:24:49 JST 1993
 
   Copyright (C) 1993-2003 Yukihiro Matsumoto
@@ -643,6 +643,7 @@ make_regexp(s, len, flags)
     if (err != NULL) {
 	re_free_pattern(rp);
 	rb_reg_raise(s, len, err, 0);
+	return 0;
     }
     return rp;
 }
@@ -679,6 +680,7 @@ match_alloc(klass)
     return (VALUE)match;
 }
 
+/* :nodoc: */
 static VALUE
 match_init_copy(obj, orig)
     VALUE obj, orig;
@@ -1973,6 +1975,7 @@ rb_reg_s_union(argc, argv)
     }
 }
 
+/* :nodoc: */
 static VALUE
 rb_reg_init_copy(copy, re)
     VALUE copy, re;
