@@ -157,6 +157,7 @@ static struct {
     {	ERROR_NEGATIVE_SEEK,		EINVAL		},
     {	ERROR_SEEK_ON_DEVICE,		EACCES		},
     {	ERROR_DIR_NOT_EMPTY,		ENOTEMPTY	},
+    {	ERROR_DIRECTORY,		ENOTDIR		},
     {	ERROR_NOT_LOCKED,		EACCES		},
     {	ERROR_BAD_PATHNAME,		ENOENT		},
     {	ERROR_MAX_THRDS_REACHED,	EAGAIN		},
@@ -247,7 +248,8 @@ static struct {
 /* interrupt stuff */
 static HANDLE interrupted_event;
 
-HANDLE GetCurrentThreadHandle(void)
+HANDLE
+GetCurrentThreadHandle(void)
 {
     static HANDLE current_process_handle = NULL;
     HANDLE h;
@@ -365,6 +367,11 @@ NtInitialize(int *argc, char ***argv)
     WORD version;
     int ret;
 
+    //
+    // subvert cmd.exe's feeble attempt at command line parsing
+    //
+    *argc = rb_w32_cmdvector(GetCommandLine(), argv);
+
     //
     // Now set up the correct time stuff
     //
@@ -373,9 +380,15 @@ NtInitialize(int *argc, char ***argv)
 
     // Initialize Winsock
     StartSockets();
+
+#ifdef _WIN32_WCE
+    // free commandline buffer
+    wce_FreeCommandLine();
+#endif
 }
 
-char *getlogin()
+char *
+getlogin()
 {
     char buffer[200];
     DWORD len = 200;
@@ -398,7 +411,7 @@ char *getlogin()
 
 static struct ChildRecord {
     HANDLE hProcess;	/* process handle */
-    pid_t pid;		/* process id */
+    rb_pid_t pid;	/* process id */
 } ChildRecord[MAXCHILDNUM];
 
 #define FOREACH_CHILD(v) do { \
@@ -416,7 +429,7 @@ FindFirstChildSlot(void)
 }
 
 static struct ChildRecord *
-FindChildSlot(pid_t pid)
+FindChildSlot(rb_pid_t pid)
 {
 
     FOREACH_CHILD(child) {
@@ -450,7 +463,8 @@ FindFreeChildSlot(void)
 }
 
 
-int SafeFree(char **vec, int vecc)
+int
+SafeFree(char **vec, int vecc)
 {
     //   vec
     //   |
@@ -578,7 +592,7 @@ rb_w32_get_osfhandle(int fh)
     return _get_osfhandle(fh);
 }
 
-pid_t
+rb_pid_t
 pipe_exec(char *cmd, int mode, FILE **fpr, FILE **fpw)
 {
     struct ChildRecord* child;
@@ -718,9 +732,7 @@ pipe_exec(char *cmd, int mode, FILE **fpr, FILE **fpw)
 extern VALUE rb_last_status;
 
 int
-do_spawn(mode, cmd)
-int mode;
-char *cmd;
+do_spawn(int mode, char *cmd)
 {
     struct ChildRecord *child;
     DWORD exitcode;
@@ -757,10 +769,7 @@ char *cmd;
 }
 
 int
-do_aspawn(mode, prog, argv)
-int mode;
-char *prog;
-char **argv;
+do_aspawn(int mode, char *prog, char **argv)
 {
     char *cmd, *p, *q, *s, **t;
     int len, n, bs, quote;
@@ -993,7 +1002,7 @@ CreateChild(const char *cmd, const char *prog, SECURITY_ATTRIBUTES *psa,
     CloseHandle(aProcessInformation.hThread);
 
     child->hProcess = aProcessInformation.hProcess;
-    child->pid = (pid_t)aProcessInformation.dwProcessId;
+    child->pid = (rb_pid_t)aProcessInformation.dwProcessId;
 
     if (!IsWinNT()) {
 	/* On Win9x, make pid positive similarly to cygwin and perl */
@@ -1018,7 +1027,7 @@ typedef struct _NtCmdLineElement {
 #define NTMALLOC 0x2	// string in element was malloc'ed
 #define NTSTRING 0x4	// element contains a quoted string
 
-static void
+static int
 insert(const char *path, VALUE vinfo)
 {
     NtCmdLineElement *tmpcurr;
@@ -1032,6 +1041,8 @@ insert(const char *path, VALUE vinfo)
     strcpy(tmpcurr->str, path);
     **tail = tmpcurr;
     *tail = &tmpcurr->next;
+
+    return 0;
 }
 
 #ifdef HAVE_SYS_PARAM_H
@@ -1056,7 +1067,7 @@ cmdglob(NtCmdLineElement *patt, NtCmdLineElement **tail)
     for (p = buf; *p; p = CharNext(p))
 	if (*p == '\\')
 	    *p = '/';
-    rb_globi(buf, insert, (VALUE)&tail);
+    ruby_globi(buf, 0, insert, (VALUE)&tail);
     if (buf != buffer)
 	free(buf);
 
@@ -1694,6 +1705,15 @@ rb_w32_strerror(int e)
     DWORD source = 0;
     char *p;
 
+#if defined __BORLANDC__ && defined ENOTEMPTY // _sys_errlist is broken
+    switch (e) {
+      case ENAMETOOLONG:
+	return "Filename too long";
+      case ENOTEMPTY:
+	return "Directory not empty";
+    }
+#endif
+
     if (e < 0 || e > sys_nerr) {
 	if (e < 0)
 	    e = GetLastError();
@@ -1702,13 +1722,17 @@ rb_w32_strerror(int e)
 			  buffer, 512, NULL) == 0) {
 	    strcpy(buffer, "Unknown Error");
 	}
-	for (p = buffer + strlen(buffer) - 1; buffer <= p; p--) {
-	    if (*p != '\r' && *p != '\n') break;
-	    *p = 0;
-	}
-	return buffer;
     }
-    return strerror(e);
+    else {
+	strncpy(buffer, strerror(e), sizeof(buffer));
+	buffer[sizeof(buffer) - 1] = 0;
+    }
+
+    for (p = buffer + strlen(buffer) - 1; buffer <= p; p--) {
+	if (*p != '\r' && *p != '\n') break;
+	*p = 0;
+    }
+    return buffer;
 }
 
 //
@@ -1725,38 +1749,38 @@ rb_w32_strerror(int e)
 #define ROOT_UID	0
 #define ROOT_GID	0
 
-UIDTYPE
+rb_uid_t
 getuid(void)
 {
 	return ROOT_UID;
 }
 
-UIDTYPE
+rb_uid_t
 geteuid(void)
 {
 	return ROOT_UID;
 }
 
-GIDTYPE
+rb_gid_t
 getgid(void)
 {
 	return ROOT_GID;
 }
 
-GIDTYPE
+rb_gid_t
 getegid(void)
 {
     return ROOT_GID;
 }
 
 int
-setuid(int uid)
+setuid(rb_uid_t uid)
 { 
     return (uid == ROOT_UID ? 0 : -1);
 }
 
 int
-setgid(int gid)
+setgid(rb_gid_t gid)
 {
     return (gid == ROOT_GID ? 0 : -1);
 }
@@ -1773,6 +1797,7 @@ int
   ioctl(int i, unsigned int u, long data)
 #endif
 {
+    errno = EINVAL;
     return -1;
 }
 
@@ -2380,9 +2405,9 @@ void endservent() {}
 
 struct netent *getnetent (void) {return (struct netent *) NULL;}
 
-struct netent *getnetbyaddr(char *name) {return (struct netent *)NULL;}
+struct netent *getnetbyaddr(long net, int type) {return (struct netent *)NULL;}
 
-struct netent *getnetbyname(long net, int type) {return (struct netent *)NULL;}
+struct netent *getnetbyname(char *name) {return (struct netent *)NULL;}
 
 struct protoent *getprotoent (void) {return (struct protoent *) NULL;}
 
@@ -2437,7 +2462,7 @@ fcntl(int fd, int cmd, ...)
 #define WNOHANG -1
 #endif
 
-static pid_t
+static rb_pid_t
 poll_child_status(struct ChildRecord *child, int *stat_loc)
 {
     DWORD exitcode;
@@ -2455,7 +2480,7 @@ poll_child_status(struct ChildRecord *child, int *stat_loc)
     }
     if (exitcode != STILL_ACTIVE) {
 	/* If already died, return immediatly. */
-	pid_t pid = child->pid;
+	rb_pid_t pid = child->pid;
 	CloseChildHandle(child);
 	if (stat_loc) *stat_loc = exitcode << 8;
 	return pid;
@@ -2463,8 +2488,8 @@ poll_child_status(struct ChildRecord *child, int *stat_loc)
     return 0;
 }
 
-pid_t
-waitpid (pid_t pid, int *stat_loc, int options)
+rb_pid_t
+waitpid (rb_pid_t pid, int *stat_loc, int options)
 {
     DWORD timeout;
 
@@ -2549,9 +2574,7 @@ gettimeofday(struct timeval *tv, struct timezone *tz)
 }
 
 char *
-rb_w32_getcwd(buffer, size)
-    char *buffer;
-    int size;
+rb_w32_getcwd(char *buffer, int size)
 {
     int length;
     char *bp;
@@ -2878,7 +2901,8 @@ rb_w32_times(struct tms *tmbuf)
 #define yield_once() Sleep(0)
 #define yield_until(condition) do yield_once(); while (!(condition))
 
-static DWORD wait_events(HANDLE event, DWORD timeout)
+static DWORD
+wait_events(HANDLE event, DWORD timeout)
 {
     HANDLE events[2];
     int count = 0;
@@ -2899,7 +2923,8 @@ static DWORD wait_events(HANDLE event, DWORD timeout)
     return ret;
 }
 
-static CRITICAL_SECTION* system_state(void)
+static CRITICAL_SECTION *
+system_state(void)
 {
     static int initialized = 0;
     static CRITICAL_SECTION syssect;
@@ -2914,7 +2939,8 @@ static CRITICAL_SECTION* system_state(void)
 static LONG flag_interrupt = -1;
 static volatile DWORD tlsi_interrupt = TLS_OUT_OF_INDEXES;
 
-void rb_w32_enter_critical(void)
+void
+rb_w32_enter_critical(void)
 {
     if (IsWinNT()) {
 	EnterCriticalSection(system_state());
@@ -2935,7 +2961,8 @@ void rb_w32_enter_critical(void)
     }
 }
 
-void rb_w32_leave_critical(void)
+void
+rb_w32_leave_critical(void)
 {
     if (IsWinNT()) {
 	LeaveCriticalSection(system_state());
@@ -2954,7 +2981,8 @@ struct handler_arg_t {
     HANDLE handshake;
 };
 
-static void rb_w32_call_handler(struct handler_arg_t* h)
+static void
+rb_w32_call_handler(struct handler_arg_t* h)
 {
     int status;
     RUBY_CRITICAL(rb_protect((VALUE (*)(VALUE))h->handler, (VALUE)h->arg, &h->status);
@@ -2967,10 +2995,9 @@ static void rb_w32_call_handler(struct handler_arg_t* h)
     yield_until(0);
 }
 
-static struct handler_arg_t* setup_handler(struct handler_arg_t *harg,
-					   int arg,
-					   void (*handler)(int),
-					   HANDLE handshake)
+static struct handler_arg_t *
+setup_handler(struct handler_arg_t *harg, int arg, void (*handler)(int),
+	      HANDLE handshake)
 {
     harg->handler = handler;
     harg->arg = arg;
@@ -2980,7 +3007,8 @@ static struct handler_arg_t* setup_handler(struct handler_arg_t *harg,
     return harg;
 }
 
-static void setup_call(CONTEXT* ctx, struct handler_arg_t *harg)
+static void
+setup_call(CONTEXT* ctx, struct handler_arg_t *harg)
 {
 #ifdef _M_IX86
     DWORD *esp = (DWORD *)ctx->Esp;
@@ -2995,7 +3023,8 @@ static void setup_call(CONTEXT* ctx, struct handler_arg_t *harg)
 #endif
 }
 
-int rb_w32_main_context(int arg, void (*handler)(int))
+int
+rb_w32_main_context(int arg, void (*handler)(int))
 {
     static HANDLE interrupt_done = NULL;
     struct handler_arg_t harg;
@@ -3056,7 +3085,8 @@ int rb_w32_main_context(int arg, void (*handler)(int))
     return TRUE;
 }
 
-int rb_w32_sleep(unsigned long msec)
+int
+rb_w32_sleep(unsigned long msec)
 {
     DWORD ret;
     RUBY_CRITICAL(ret = wait_events(NULL, msec));
@@ -3065,7 +3095,8 @@ int rb_w32_sleep(unsigned long msec)
     return ret != WAIT_TIMEOUT;
 }
 
-static void catch_interrupt(void)
+static void
+catch_interrupt(void)
 {
     yield_once();
     RUBY_CRITICAL(wait_events(NULL, 0));
@@ -3073,7 +3104,8 @@ static void catch_interrupt(void)
 }
 
 #undef fgetc
-int rb_w32_getc(FILE* stream)
+int
+rb_w32_getc(FILE* stream)
 {
     int c, trap_immediate = rb_trap_immediate;
 #ifndef _WIN32_WCE
@@ -3097,7 +3129,8 @@ int rb_w32_getc(FILE* stream)
 }
 
 #undef fputc
-int rb_w32_putc(int c, FILE* stream)
+int
+rb_w32_putc(int c, FILE* stream)
 {
     int trap_immediate = rb_trap_immediate;
 #ifndef _WIN32_WCE
@@ -3204,7 +3237,8 @@ rb_w32_asynchronize(asynchronous_func_t func, VALUE self,
     return val;
 }
 
-char **rb_w32_get_environ(void)
+char **
+rb_w32_get_environ(void)
 {
     char *envtop, *env;
     char **myenvtop, **myenv;
@@ -3237,7 +3271,8 @@ char **rb_w32_get_environ(void)
     return myenvtop;
 }
 
-void rb_w32_free_environ(char **env)
+void
+rb_w32_free_environ(char **env)
 {
     char **t = env;
 
@@ -3246,9 +3281,10 @@ void rb_w32_free_environ(char **env)
 }
 
 #undef getpid
-pid_t rb_w32_getpid(void)
+rb_pid_t
+rb_w32_getpid(void)
 {
-    pid_t pid;
+    rb_pid_t pid;
 
     pid = getpid();
 
@@ -3300,8 +3336,9 @@ unixtime_to_filetime(time_t time, FILETIME *ft)
 {
     struct tm *tm;
     SYSTEMTIME st;
+    FILETIME lt;
 
-    tm = gmtime(&time);
+    tm = localtime(&time);
     st.wYear = tm->tm_year + 1900;
     st.wMonth = tm->tm_mon + 1;
     st.wDayOfWeek = tm->tm_wday;
@@ -3310,17 +3347,14 @@ unixtime_to_filetime(time_t time, FILETIME *ft)
     st.wMinute = tm->tm_min;
     st.wSecond = tm->tm_sec;
     st.wMilliseconds = 0;
-    if (!SystemTimeToFileTime(&st, ft)) {
+    if (!SystemTimeToFileTime(&st, &lt) ||
+	!LocalFileTimeToFileTime(&lt, ft)) {
 	errno = map_errno(GetLastError());
 	return -1;
     }
     return 0;
 }
 
-#undef utime
-#ifdef __BORLANDC__
-#define utime _utime
-#endif
 int
 rb_w32_utime(const char *path, struct utimbuf *times)
 {
@@ -3381,19 +3415,17 @@ rb_w32_snprintf(char *buf, size_t size, const char *format, ...)
     return ret;
 }
 
-#undef mkdir
-#undef rmdir
 int
 rb_w32_mkdir(const char *path, int mode)
 {
     int ret = -1;
     RUBY_CRITICAL(do {
-	if (mkdir(path) == -1)
+	if (CreateDirectory(path, NULL) == FALSE) {
+	    errno = map_errno(GetLastError());
 	    break;
+	}
 	if (chmod(path, mode) == -1) {
-	    int save_errno = errno;
-	    rmdir(path);
-	    errno = save_errno;
+	    RemoveDirectory(path);
 	    break;
 	}
 	ret = 0;
@@ -3404,17 +3436,38 @@ rb_w32_mkdir(const char *path, int mode)
 int
 rb_w32_rmdir(const char *path)
 {
-    DWORD attr;
-    int ret;
+    int ret = 0;
     RUBY_CRITICAL({
-	attr = GetFileAttributes(path);
+	const DWORD attr = GetFileAttributes(path);
 	if (attr != (DWORD)-1 && (attr & FILE_ATTRIBUTE_READONLY)) {
-	    attr &= ~FILE_ATTRIBUTE_READONLY;
-	    SetFileAttributes(path, attr);
+	    SetFileAttributes(path, attr & ~FILE_ATTRIBUTE_READONLY);
 	}
-	ret = rmdir(path);
-	if (ret < 0 && attr != (DWORD)-1) {
-	    SetFileAttributes(path, attr);
+	if (RemoveDirectory(path) == FALSE) {
+	    errno = map_errno(GetLastError());
+	    ret = -1;
+	    if (attr != (DWORD)-1 && (attr & FILE_ATTRIBUTE_READONLY)) {
+		SetFileAttributes(path, attr);
+	    }
+	}
+    });
+    return ret;
+}
+
+int
+rb_w32_unlink(const char *path)
+{
+    int ret = 0;
+    RUBY_CRITICAL({
+	const DWORD attr = GetFileAttributes(path);
+	if (attr != (DWORD)-1 && (attr & FILE_ATTRIBUTE_READONLY)) {
+	    SetFileAttributes(path, attr & ~FILE_ATTRIBUTE_READONLY);
+	}
+	if (DeleteFile(path) == FALSE) {
+	    errno = map_errno(GetLastError());
+	    ret = -1;
+	    if (attr != (DWORD)-1 && (attr & FILE_ATTRIBUTE_READONLY)) {
+		SetFileAttributes(path, attr);
+	    }
 	}
     });
     return ret;
